@@ -1,11 +1,15 @@
 import json
 import os
 from datetime import datetime
+import re
+import pathlib
+from pathlib import Path
 from pprint import pprint
 import numpy as np
 import cv2
 import pygame as pg
-from hexss.image.func import pygame_surface_to_numpy
+from hexss.box import Box
+from hexss.image.pygame import pygame_surface_to_numpy
 from pygame import Rect, Surface, mouse, MOUSEBUTTONDOWN
 import pygame_gui
 from pygame_gui import UIManager, UI_FILE_DIALOG_PATH_PICKED, UI_BUTTON_PRESSED, UI_DROP_DOWN_MENU_CHANGED, \
@@ -15,14 +19,27 @@ from pygame_gui.elements import UIPanel, UILabel, UIButton, UIDropDownMenu, UISe
 from pygame_gui.windows import UIFileDialog
 from keras import models
 from hexss.constants.terminal_color import *
-from .adj_image import adj_image
 from hexss import json_load, json_update
-from hexss.image import get_image, crop_img
+from hexss.image import get_image, crop_img, Image
 from .theme import theme
 from .textbox_surface import TextBoxSurface, gradient_surface
 from .pygame_function import putText, UITextBox
 from os.path import join
 from .summary_graphs import summary
+from PIL import Image as PILImage
+
+
+def pil_to_numpy_strict(img):
+    if hasattr(img, "image") and not isinstance(img, PILImage.Image):
+        img = img.image
+
+    w, h = img.size
+    bands = len(img.getbands())  # 1 for L, 3 for RGB, 4 for RGBA
+    buf = np.frombuffer(img.tobytes(), dtype=np.uint8)
+    if bands == 1:
+        return buf.reshape(h, w)
+    else:
+        return buf.reshape(h, w, bands)
 
 
 class RightClick:
@@ -47,6 +64,8 @@ class RightClick:
         if len(options_list) > 0:
             selection_size = (max_character * 8 + 20, 6 + 25 * len(options_list))
             pos = np.minimum(mouse_pos, self.window_size - selection_size)
+            options_list = list(options_list)
+            options_list.sort(),
             self.selection = UISelectionList(
                 Rect(pos, selection_size),
                 item_list=options_list,
@@ -95,10 +114,10 @@ class AutoInspection:
         self.img_surface = pg.image.frombuffer(np_img.tobytes(), np_img.shape[1::-1], "BGR")
         self.update_scaled_img_surface()
 
-    def get_surface_form_url(self):
-        url = "http://127.0.0.1:2002/image?source=video_capture&id=0"
-        self.np_img = get_image(url, 'numpy')
-        self.get_surface_form_np(self.np_img)
+    def get_surface_form_config_data(self):
+        if self.data.get('img') is not None:
+            self.np_img = self.data.get('img')
+            self.get_surface_form_np(self.np_img)
 
     def show_rects_to_surface(self, frame_dict, type='frame'):
         if self.is_show_rects:
@@ -167,6 +186,7 @@ class AutoInspection:
         self.data['model_name'] = self.config['model_name']
 
         self.np_img = self.IMG.copy()
+        self.mark_template_im = None
         # if ui:
         pg.init()
         pg.display.set_caption('Auto Inspection')
@@ -184,6 +204,8 @@ class AutoInspection:
         self.pass_n = 0
         self.fail_n = 0
 
+        self.buff = '----------'
+
     def set_name_for_debug(self, file_name=None):
         self.file_name = datetime.now().strftime("%y%m%d %H%M%S") if file_name is None else file_name
         json_path = join(self.model_name_dir(), 'img_full', f'{self.file_name}.json')
@@ -198,6 +220,7 @@ class AutoInspection:
             if not frame.get('frame_name'):
                 frame['frame_name'] = name
         self.res_textbox.update_text('res', text='-', color=(0, 0, 0))
+        self.config['res'] = '-'
         self.setup_NG_details()
 
     def setup_NG_details(self):
@@ -206,7 +229,7 @@ class AutoInspection:
         for k, v in self.frame_dict.items() if self.frame_dict else ():
             if v.get('highest_score_name') in ['', 'OK']:
                 continue
-            text = f"{k} {v['highest_score_name']} {v['highest_score_percent']}"
+            text = f"{k} {v['highest_score_name']}"
             formatted_text += f"<font color='#FF0000' size={size_font}>{text}</font><br>"
         self.res_NG_text_box.set_text(formatted_text)
 
@@ -231,11 +254,15 @@ class AutoInspection:
             frames_pos_file_path = join(self.model_name_dir(), 'frames pos.json')
             json_data = json_load(frames_pos_file_path, {})
 
-            self.frame_dict = json_data.get('frames')
-            self.model_dict = json_data.get('models')
-            self.mark_dict = json_data.get('marks')
+            self.frame_dict: dict = json_data.get('frames')
+            self.model_dict: dict = json_data.get('models')
+            self.mark_dict: dict = json_data.get('marks')
             self.reset_frame()
 
+            mark_template_path = Path(self.model_name_dir()) / 'mark_template.png'
+            if mark_template_path.exists():
+                self.mark_template_im = Image(mark_template_path)
+
             for name, frame in self.mark_dict.items() if self.mark_dict else ():
                 frame['color_rect'] = (200, 20, 100)
                 frame['width_rect'] = 1
@@ -250,7 +277,10 @@ class AutoInspection:
                           f'file error <data>/{self.data["model_name"]}/model/{name}.h5{END}')
                     print(PINK, e, END, sep='')
                 try:
-                    model.update(json_load(join(self.model_name_dir(), f'model/{name}.json')))
+                    json_file_with_model = json_load(join(self.model_name_dir(), f'model/{name}.json'))
+                    if json_file_with_model.get('class_names'):
+                        json_file_with_model['model_class_names'] = json_file_with_model.pop('class_names')
+                    model.update(json_file_with_model)
                     pprint(model)
                     if model['model_class_names'] != model['class_names']:
                         print(f'{YELLOW}class_names       = {model["class_names"]}')
@@ -276,10 +306,16 @@ class AutoInspection:
             self.predict_button.enable() if self.model_dict else self.predict_button.disable()
 
         self.res_textbox.update_text('res', text='-')
+        self.config['res'] = '-'
         self.setup_NG_details()
 
+        if self.model_data_dropdown:
+            self.model_data_dropdown.kill()
+        self.create_model_data_dropdown(self.data['model_name'])
+
     def predict(self):
         self.res_textbox.update_text('res', text='Wait', color=(255, 255, 0))
+        self.config['res'] = 'Wait'
         self.manager.draw_ui(self.display)
         pg.display.update()
 
@@ -323,9 +359,11 @@ class AutoInspection:
         if res_surface_text == 'OK':
             self.pass_n += 1
             self.res_textbox.update_text('res', text='OK', color=(0, 255, 0))
+            self.config['res'] = 'OK'
         else:
             self.fail_n += 1
             self.res_textbox.update_text('res', text='NG', color=(255, 0, 0))
+            self.config['res'] = 'NG'
         self.update_status()
         self.save_result = 1
 
@@ -378,6 +416,14 @@ class AutoInspection:
             object_id=ObjectID(class_id='@minimize_button', object_id='#minimize_button'),
             anchors=anchors | {'right_target': self.close_button}
         )
+        if self.data['config'].get('fake_version'):
+            UILabel(
+                Rect(-100, 0, 100, 40),
+                f'{self.data["config"].get("fake_version")}',
+                self.manager,
+                object_id=ObjectID(class_id='@model_label', object_id='#model_label'),
+                anchors=anchors | {'right_target': self.minimize_button}
+            )
 
         # bottom left
         anchors = {'top': 'bottom', 'left': 'left', 'bottom': 'bottom', 'right': 'left'}
@@ -408,7 +454,9 @@ class AutoInspection:
 
         # bottom left
         anchors = {'top': 'bottom', 'left': 'right', 'bottom': 'bottom', 'right': 'right'}
-        test = f'Auto Inspection 0.2.2' + f" x {self.xfunction}" if self.xfunction else ''
+        init_txt = pathlib.Path(__file__).with_name('__init__.py').read_text(encoding='utf-8')
+        version = re.search(r"__version__\s*=\s*['\"]([^'\"]+)['\"]", init_txt).group(1)
+        test = f'Auto Inspection {version}' + (f" x {self.xfunction}" if self.xfunction else '')
         w = (150 / 22 * len(test)) + 10
         self.autoinspection_button = UIButton(
             Rect(-w, -30, w, 30) if is_full_hd else Rect(-w, -20, w, 20),
@@ -714,7 +762,7 @@ class AutoInspection:
     def panel2_update(self, events):
 
         def capture_button():
-            self.get_surface_form_url()
+            self.get_surface_form_config_data()
             self.reset_frame()
             self.set_name_for_debug()
 
@@ -722,7 +770,31 @@ class AutoInspection:
             self.auto_cap_button.set_text('Auto' if self.auto_cap_button.text == 'Stop' else 'Stop')
 
         def adj_button():
-            self.np_img = adj_image(self.np_img, self.data['model_name'], self.mark_dict)
+            if self.mark_template_im is None:
+                return
+
+            im2 = Image(self.np_img.copy())
+
+            for mark in self.mark_dict.values():
+                box = Box(xywhn=mark['xywh'])
+                box.set_size(self.mark_template_im.size)
+                mark['im'] = self.mark_template_im.crop(box)
+                scale_xywhn = box.scale(mark.get('k', 3)).xywhn
+                mark['scale_xywhn'] = scale_xywhn
+
+            pts_src = []
+            pts_dst = []
+            for mark in self.mark_dict.values():
+                xywhn_mark_area = mark['scale_xywhn']
+                mark_im = mark['im']
+                xy, score = im2.best_match_location(mark_im, xywhn=xywhn_mark_area)
+                pts_dst.append(Box(size=im2.size, xywhn=mark['xywh']).xy)
+                pts_src.append(xy)
+
+            im2.align_image(pts_src, pts_dst)
+            arr = pil_to_numpy_strict(im2.image)
+            self.np_img = cv2.cvtColor(arr, cv2.COLOR_RGB2BGR)
+
             self.reset_frame()
 
         is_full_hd = self.resolution == '1920x1080'
@@ -739,7 +811,14 @@ class AutoInspection:
                 self.predict()
             if event == 'Capture&Predict':
                 capture_button()
+                adj_button()
                 self.predict()
+            if event == 'change_model':
+                self.change_model()
+            if event == 'change_image':
+                self.get_surface_form_config_data()
+                self.reset_frame()
+                self.set_name_for_debug()
 
         for event in events:
             if event.type == UI_BUTTON_PRESSED:
@@ -765,6 +844,7 @@ class AutoInspection:
                     self.predict()
                 if event.ui_element == self.capture_predict_button:
                     capture_button()
+                    adj_button()
                     self.predict()
             if event.type == UI_FILE_DIALOG_PATH_PICKED:
                 # from Load Image
@@ -791,7 +871,7 @@ class AutoInspection:
             #         self.set_name_for_debug()
 
         if self.auto_cap_button.text == 'Stop':
-            self.get_surface_form_url()
+            self.get_surface_form_config_data()
 
     def setup_ui(self, ):
         self.panel0_setup()
@@ -812,6 +892,21 @@ class AutoInspection:
             if event.type == pygame_gui.UI_BUTTON_ON_UNHOVERED:
                 self.manager.set_active_cursor(pg.SYSTEM_CURSOR_ARROW)
 
+            if event.type == 768:
+                if event.dict.get('unicode') and event.unicode in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-<>':
+                    self.buff += event.unicode
+                    self.buff = self.buff[-20:]
+
+                    # <FM-R511-000>
+                    if self.buff[-1] == '>':
+                        model_name = self.buff[:-1]
+                        model_name = model_name.split('<')[-1]
+                        print(model_name)
+                        self.buff += '-'
+
+                        self.data['model_name'] = model_name
+                        self.change_model()
+
         self.right_click.events(events)
 
     def run(self):
